#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Utilities and classes for unbundling and archiving a tar file."""
from __future__ import print_function
import tarfile
import json
import hashlib
import time
import os
from six import PY2
import requests
from pacifica.ingest.utils import get_unique_id


class HashValidationException(Exception):
    """Class to capture hashsum validation failures."""

    pass


class FileIngester(object):
    """Class to ingest a single file from a tar file into the file archives."""

    fileobj = None
    file_id = 0
    recorded_hash = ''
    hashval = None
    server = ''

    def __init__(self, hashtype, hashcode, server, file_id):
        """Constructor for FileIngester class."""
        if hashtype in hashlib.algorithms_available:
            self.hashval = getattr(hashlib, hashtype)()
        else:
            raise ValueError('Invalid Hashtype {}'.format(hashtype))
        self.recorded_hash = hashcode
        self.server = server
        self.file_id = file_id
        self.session = requests.session()
        retry_adapter = requests.adapters.HTTPAdapter(max_retries=5)
        self.session.mount('https://', retry_adapter)
        self.session.mount('http://', retry_adapter)

    def read(self, size):
        """Read wrapper for requests that calculates the hashcode inline."""
        buf = self.fileobj.read(size)
        # running checksum
        self.hashval.update(buf)
        return buf

    def validate_hash(self):
        """Validate that the calculated hash matches the hash uploaded in the tar file."""
        file_hash = self.hashval.hexdigest()
        if self.recorded_hash == file_hash:
            return True
        return False

    def upload_file_in_file(self, info, tar):
        """Upload a file from inside a tar file."""
        self.fileobj = tar.extractfile(info)
        size = info.size
        size_str = str(size)
        mod_time = time.ctime(info.mtime)
        self.fileobj.seek(0)
        url = self.server + str(self.file_id)

        headers = {}
        headers['Last-Modified'] = mod_time
        headers['Content-Type'] = 'application/octet-stream'
        headers['Content-Length'] = size_str

        # pylint: disable=assignment-from-no-return
        req = self.session.put(
            url,
            data=self,
            headers=headers
        )
        # pylint: enable=assignment-from-no-return
        self.fileobj.close()
        body = req.text
        ret_dict = json.loads(body)
        size = int(ret_dict['total_bytes'])
        if size != info.size:  # pragma: no cover
            return False
        success = self.validate_hash()
        print('validated = ' + str(success))
        if not success:
            # roll back upload
            raise HashValidationException(
                'File {} failed to validate.'.format(self.file_id))
        return True


class MetaParser(object):
    """Class used to hold and search metadata."""

    # entire metadata
    meta = None
    # a map of filenames to hashcodes
    files = {}
    start_id = -999
    transaction_id = -999
    file_count = -999

    meta_str = ''

    def __init__(self):
        """Constructor."""
        self.session = requests.session()
        retry_adapter = requests.adapters.HTTPAdapter(max_retries=5)
        self.session.mount('https://', retry_adapter)
        self.session.mount('http://', retry_adapter)

    def file_obj_count(self, meta_list):
        """Count the file objects in metadata and keep the count."""
        self.file_count = 0
        for meta in meta_list:
            if meta['destinationTable'] == 'Files':
                self.file_count += 1

    def read_meta(self, metafile, job_id):
        """Read the metadata from metafile and assume it's good."""
        self.transaction_id = job_id
        meta_list = json.loads(open(metafile).read())
        self.file_obj_count(meta_list)
        self.start_id = get_unique_id(self.file_count, 'file')
        self.files = {}
        # all we care about for now is the hash and the file path
        file_id = self.start_id
        for meta in meta_list:
            if meta['destinationTable'] == 'Files':
                meta['_id'] = file_id
                self.files[str(file_id)] = meta
                file_id += 1
        trans = {}
        trans['destinationTable'] = 'Transactions._id'
        trans['value'] = self.transaction_id
        meta_list.append(trans)
        self.meta_str = json.dumps(meta_list, sort_keys=True, indent=4)

    def load_meta(self, tar, job_id):
        """Load the metadata from a tar file into searchable structures."""
        # transaction id is the unique upload job id created by the ingest frontend
        self.transaction_id = job_id

        string = tar.extractfile('metadata.txt').read()
        uni_str = string if PY2 else string.decode('utf8')
        meta_list = json.loads(uni_str)

        # get the start index for the file
        self.file_count = file_count(tar)
        self.start_id = get_unique_id(self.file_count, 'file')

        self.files = {}

        # all we care about for now is the hash and the file path
        file_id = self.start_id
        for meta in meta_list:
            if meta['destinationTable'] == 'Files':
                meta['_id'] = file_id
                self.files[str(file_id)] = meta
                file_id += 1

        trans = {}
        trans['destinationTable'] = 'Transactions._id'
        trans['value'] = self.transaction_id
        meta_list.append(trans)

        self.meta_str = json.dumps(meta_list, sort_keys=True, indent=4)

    def get_hash(self, file_id):
        """Return the hash string for a file name."""
        file_element = self.files[file_id]
        # remove filetype if there is one
        file_hash = file_element['hashsum'].replace('sha1:', '')
        file_hash_type = file_element['hashtype']
        return file_hash_type, file_hash

    def get_fname(self, file_id):
        """Get the file name from the file ID."""
        file_element = self.files[file_id]
        name = file_element['name']
        return name

    def get_subdir(self, file_id):
        """Get the sub directory element from the file ID."""
        file_element = self.files[file_id]
        name = file_element['subdir']
        return name

    def clean_metadata(self):
        """clean /data from filepaths."""
        meta_list = json.loads(self.meta_str)

        for meta in meta_list:
            if meta['destinationTable'] == 'Files':
                meta['subdir'] = get_clipped(meta['subdir'])

        self.meta_str = json.dumps(meta_list, sort_keys=True, indent=4)

    def post_metadata(self):
        """Upload metadata to server."""
        try:
            self.clean_metadata()

            archive_server = os.getenv('METADATA_SERVER', '127.0.0.1')
            archive_port = os.getenv('METADATA_PORT', '8121')
            archive_url = 'http://{0}:{1}/ingest'.format(
                archive_server, archive_port)

            headers = {'content-type': 'application/json'}

            # pylint: disable=assignment-from-no-return
            req = self.session.put(
                archive_url, headers=headers, data=self.meta_str)
            # pylint: enable=assignment-from-no-return
            if req.json()['status'] == 'success':
                return True, ''
        # pylint: disable=broad-except
        except Exception as ex:
            return False, ex
        # pylint: enable=broad-except
        return False, req.json()


def get_clipped(fname):
    """Return a file path with the data separator removed."""
    parts = fname.split('/')  # this is posix tar standard
    if parts[0] == 'data':
        del parts[0]
    parts = [x for x in parts if x]
    return '/'.join(parts)  # this is also posix tar standard


# pylint: disable=too-few-public-methods
class TarIngester(object):
    """Class to read a tar file and upload it to the metadata and file archives."""

    tar = None
    meta = None

    def __init__(self, tar, meta):
        """Constructor for TarIngester class."""
        self.tar = tar
        self.meta = meta

    def ingest(self):
        """Ingest a tar file into the file archive."""
        archive_server = os.getenv('ARCHIVEINTERFACE_SERVER', '127.0.0.1')
        archive_port = os.getenv('ARCHIVEINTERFACE_PORT', '8080')
        archive_url = 'http://{0}:{1}/'.format(archive_server, archive_port)

        for file_id in self.meta.files.keys():
            file_hash_type, file_hash = self.meta.get_hash(file_id)
            name = self.meta.get_fname(file_id)

            path = self.meta.get_subdir(file_id) + '/' + name
            # this is for posix tar standard
            path = '/'.join(['data', get_clipped(path)])
            uni_path = path.encode('utf-8') if PY2 else path
            info = self.tar.getmember(uni_path)
            print(info.name)
            ingest = FileIngester(
                file_hash_type, file_hash, archive_url, file_id)
            ingest.upload_file_in_file(info, self.tar)
# pylint: enable=too-few-public-methods


def open_tar(fpath):
    """Seek to the location of fpath, returns a file stream pointer and file size."""
    # check validity
    if not tarfile.is_tarfile(fpath):
        return None

    # open tar file
    try:
        tar = tarfile.open(fpath, 'r:')
    # not sure what exceptions would show up here and not be covered by is_tarfile
    except tarfile.TarError:  # pragma: no cover
        print('Error opening: ' + fpath)
        return None

    return tar


def patch_files(meta_obj):
    """Patch the files in the archive interface."""
    archive_server = os.getenv('ARCHIVEINTERFACE_SERVER', '127.0.0.1')
    archive_port = os.getenv('ARCHIVEINTERFACE_PORT', '8080')
    archive_url = 'http://{0}:{1}/'.format(archive_server, archive_port)
    session = requests.session()
    retry_adapter = requests.adapters.HTTPAdapter(max_retries=5)
    session.mount('https://', retry_adapter)
    session.mount('http://', retry_adapter)
    for file_id in meta_obj.files.keys():
        data = {'path': meta_obj.files[file_id]['source']}
        req = session.patch(
            '{}{}'.format(archive_url, file_id),
            headers={'content-type': 'application/json'},
            data=json.dumps(data)
        )
        if req.json().get('message') != 'File Moved Successfully':
            raise Exception(json.dumps(req.json()))


def file_count(tar):
    """
    Retrieve the file count for a tar file.

    Does not count metadata.txt as that is not uploaded to the file archive
    """
    members = tar.getmembers()
    # don't count the metadata.txt file
    return len(members) - 1



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""ORM for index server."""
import time
from datetime import datetime
import peewee
from playhouse.db_url import connect
from pacifica.ingest.config import get_config

DATABASE_CONNECT_ATTEMPTS = 20
DATABASE_WAIT = 5
DB = connect(get_config().get('database', 'peewee_url'))


def database_setup(attempts=0):
    """Attempt to connect to the database."""
    try:
        IngestState.database_connect()
        if not IngestState.table_exists():
            IngestState.create_table()
        IngestState.database_close()
    except peewee.OperationalError:
        if attempts < DATABASE_CONNECT_ATTEMPTS:
            time.sleep(DATABASE_WAIT)
            attempts += 1
            database_setup(attempts)
        else:
            raise peewee.OperationalError


# pylint: disable=too-few-public-methods
class BaseModel(peewee.Model):
    """Auto-generated by pwiz."""

    class Meta(object):
        """Map to the database connected above."""

        database = DB


class IngestState(BaseModel):
    """Map a python record to a mysql table."""

    job_id = peewee.BigIntegerField(primary_key=True, db_column='id')
    state = peewee.CharField()
    task = peewee.CharField()
    task_percent = peewee.DecimalField()
    exception = peewee.TextField(default='')
    created = peewee.DateTimeField(default=datetime.utcnow)
    updated = peewee.DateTimeField(default=datetime.utcnow)

    @classmethod
    def database_connect(cls):
        """
        Make sure database is connected.

        Trying to connect a second
        time *does* cause problems.
        """
        # pylint: disable=no-member
        if not cls._meta.database.is_closed():
            cls._meta.database.close()
        cls._meta.database.connect()
        # pylint: enable=no-member

    @classmethod
    def database_close(cls):
        """
        Close the database connection.

        Closing already closed database
        is not a problem, so continue on.
        """
        # pylint: disable=no-member
        if not cls._meta.database.is_closed():
            cls._meta.database.close()
        # pylint: enable=no-member

    class Meta(object):
        """Map to uniqueindex table."""

        db_table = 'ingeststate'
# pylint: enable=too-few-public-methods


def update_state(job_id, state, task, task_percent, exception=''):
    """Update the state of an ingest job."""
    if job_id and int(job_id) >= 0:
        IngestState.database_connect()
        record = IngestState.get_or_create(job_id=job_id,
                                           defaults={'task': '', 'task_percent': 0, 'state': ''})[0]

        record.state = state
        record.task = task
        record.task_percent = task_percent
        record.exception = exception
        record.updated = datetime.utcnow()
        record.save()
        IngestState.database_close()


def read_state(job_id):
    """Return the state of an ingest job as a json object."""
    IngestState.database_connect()
    if job_id and job_id >= 0:
        record = IngestState.get(job_id=job_id)
    else:
        record = IngestState()
        record.state = 'DATA_ACCESS_ERROR'
        record.task = 'read_state'
        record.task_percent = 0
    IngestState.database_close()
    return record



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Ingest Server Main."""
import os
import shutil
import json
from six import PY2
import peewee
import cherrypy
from pacifica.ingest.orm import read_state, update_state
from pacifica.ingest.utils import get_unique_id, create_state_response
from pacifica.ingest.tasks import move, ingest


def error_page_default(**kwargs):
    """The default error page should always enforce json."""
    cherrypy.response.headers['Content-Type'] = 'application/json'
    return json.dumps({
        'status': kwargs['status'],
        'message': kwargs['message'],
        'traceback': kwargs['traceback'],
        'version': kwargs['version']
    })


# pylint: disable=too-few-public-methods
class RestIngestState(object):
    """The CherryPy ingest state object."""

    exposed = True

    # Cherrypy requires these named methods.
    # pylint: disable=invalid-name
    @staticmethod
    @cherrypy.tools.json_out()
    def GET(job_id):
        """Get the ingest state for the job."""
        try:
            record = read_state(int(job_id))
        except peewee.DoesNotExist:
            raise cherrypy.HTTPError(
                '404 Not Found', 'job ID {} does not exist.'.format(job_id))
        return create_state_response(record)
    # pylint: enable=invalid-name


class RestMove(object):
    """Ingest the data from the service."""

    exposed = True

    # Cherrypy requires these named methods.
    # pylint: disable=invalid-name
    @staticmethod
    @cherrypy.tools.json_out()
    @cherrypy.tools.json_in()
    def POST():
        """Post the uploaded data."""
        job_id = get_unique_id(1, 'upload_job')
        update_state(job_id, 'OK', 'UPLOADING', 0)
        root = os.getenv('VOLUME_PATH', '/tmp')
        name = str(job_id) + '.json'
        name = os.path.join(root, name)
        with open(name, 'wb') as ingest_fd:
            uni_str = json.dumps(cherrypy.request.json)
            bytes_str = uni_str if PY2 else bytes(uni_str, 'utf8')
            ingest_fd.write(bytes_str)
        move.delay(job_id, name)
        return create_state_response(read_state(job_id))
    # pylint: enable=invalid-name


class RestUpload(object):
    """Ingest the data from the service."""

    exposed = True

    # Cherrypy requires these named methods.
    # pylint: disable=invalid-name
    @staticmethod
    @cherrypy.tools.json_out()
    def POST():
        """Post the uploaded data."""
        job_id = get_unique_id(1, 'upload_job')
        update_state(job_id, 'OK', 'UPLOADING', 0)
        root = os.getenv('VOLUME_PATH', '/tmp')
        name = str(job_id) + '.tar'
        name = os.path.join(root, name)
        with open(name, 'wb') as ingest_fd:
            shutil.copyfileobj(cherrypy.request.body, ingest_fd)
        ingest.delay(job_id, name)
        return create_state_response(read_state(job_id))
    # pylint: enable=invalid-name


class Root(object):
    """The CherryPy root object."""

    exposed = False
    get_state = RestIngestState()
    upload = RestUpload()
    move = RestMove()
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Index server unit and integration tests."""
from __future__ import absolute_import
import os
from tempfile import mkstemp
import requests
from pacifica.ingest.orm import IngestState, update_state, read_state
from pacifica.ingest.utils import get_unique_id
from pacifica.ingest.tarutils import open_tar
from pacifica.ingest.tarutils import MetaParser
from pacifica.ingest.tarutils import TarIngester
from pacifica.ingest.tarutils import FileIngester
from pacifica.ingest.tasks import ingest
from ingest_db_setup_test import IngestDBSetup


class IndexServerUnitTests(IngestDBSetup):
    """Index server unit and integration tests."""

    def test_file_ingester(self):
        """Test the FileIngester class."""
        FileIngester('sha1', 'fakehashsum', '127.0.0.1', '1')
        hit_exception = False
        try:
            FileIngester('badfunc', 'fakehashsum', '127.0.0.1', '1')
        except ValueError:
            hit_exception = True
        self.assertTrue(hit_exception)

    def test_load_meta(self):
        """Test sucking metadata from uploader and configuring it in a dictionary suitable to blob to meta ingest."""
        tar = open_tar('test_data/good.tar')

        meta = MetaParser()
        meta.load_meta(tar, 1)
        self.assertTrue(meta)

    def test_tasks(self):
        """Test the ingest task."""
        job_id = get_unique_id(1, 'upload_job')

        ingest(job_id, 'test_data/good.tar')
        self.assertTrue(job_id)

    def test_post_metadata(self):
        """Test sucking metadata from uploader and configuring it in a dictionary suitable to blob to meta ingest."""
        tar = open_tar('test_data/good.tar')
        meta = MetaParser()
        meta.load_meta(tar, 1)
        success, exception = meta.post_metadata()
        self.assertTrue(success)
        self.assertFalse(exception)
        tar = open_tar('test_data/bad-mimetype.tar')
        meta = MetaParser()
        meta.load_meta(tar, 2)
        success, exception = meta.post_metadata()
        self.assertFalse(success)
        self.assertTrue(exception)

    def test_down_metadata(self):
        """Test a failed upload of the metadata."""
        tar = open_tar('test_data/good.tar')
        meta = MetaParser()

        def bad_put(*args, **kwargs):  # pylint: disable=unused-argument
            """bad put to the metadata server."""
            raise requests.HTTPError()
        meta.session.put = bad_put
        meta.load_meta(tar, 1)
        success, exception = meta.post_metadata()
        self.assertFalse(success)
        self.assertTrue(exception)

    def test_ingest_tar(self):
        """Test moving individual files to the archive files are validated inline with the upload."""
        tar = open_tar('test_data/good.tar')
        meta = MetaParser()
        meta.load_meta(tar, 1)

        tingest = TarIngester(tar, meta)
        # validate archive process

        # if not valid:
        #     rollback()
        # success = MetaUpload()
        tingest.ingest()
        self.assertTrue(tingest)

    def test_update_state(self):
        """Test return and update of unique index."""
        rwfd, fname = mkstemp()
        os.close(rwfd)
        test_object = IngestState.create(
            job_id=999,
            state='ERROR',
            task='unbundling',
            task_percent=42.3
        )
        self.assertEqual(test_object.job_id, 999)
        IngestState.database_close()
        update_state(999, 'WORKING', 'validating', 33.2)
        record = read_state(999)
        self.assertEqual(record.state, 'WORKING')
        self.assertEqual(record.task, 'validating')
        self.assertEqual(float(record.task_percent), 33.2)
        record = read_state(None)
        self.assertEqual(record.state, 'DATA_ACCESS_ERROR')
        self.assertEqual(record.task, 'read_state')
        self.assertEqual(record.task_percent, 0)
        os.unlink(fname)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test cart database setup class."""
from unittest import TestCase
from peewee import SqliteDatabase
from pacifica.ingest.orm import IngestState


class IngestDBSetup(TestCase):
    """Contain all the tests for the Cart Interface."""

    # pylint: disable=invalid-name
    def setUp(self):
        """Setup the database with in memory sqlite."""
        self._db = SqliteDatabase('file:cachedb?mode=memory&cache=shared')
        for model in [IngestState]:
            model.bind(self._db, bind_refs=False, bind_backrefs=False)
        self._db.connect()
        self._db.create_tables([IngestState])

    def tearDown(self):
        """Tear down the database."""
        self._db.drop_tables([IngestState])
        self._db.close()
        self._db = None
    # pylint: enable=invalid-name



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Testable utilities for ingest."""
from __future__ import print_function
import os
import json
import requests


def create_state_response(record):
    """Create the state response body from a record."""
    return {
        'job_id': record.job_id,
        'state': record.state,
        'task': record.task,
        'task_percent': str(record.task_percent),
        'updated': str(record.updated),
        'created': str(record.created),
        'exception': str(record.exception)
    }


def get_unique_id(id_range, mode):
    """Return a unique job id from the id server."""
    uniqueid_server = os.getenv('UNIQUEID_SERVER', '127.0.0.1')
    uniqueid_port = os.getenv('UNIQUEID_PORT', '8051')

    url = 'http://{0}:{1}/getid?range={2}&mode={3}'.format(
        uniqueid_server, uniqueid_port, id_range, mode)

    req = requests.get(url)
    body = req.text
    info = json.loads(body)
    unique_id = info['startIndex']

    return unique_id



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test ingest with a disabled archive interface."""
from __future__ import print_function, absolute_import
from common_methods_test import try_good_upload, try_good_move


# this is a long name but descriptive.
# pylint: disable=invalid-name
def test_bad_archiveinterface_upload():
    """Test if the archive interface is down."""
    try_good_upload('good', 'FAILED', 'ingest files', 0, 10)
# pylint: enable=invalid-name


def test_bad_ai_move():
    """Test the good move."""
    try_good_move('move-md', 'FAILED', 'move files', 0, 10)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Global configuration options expressed in environment variables."""
from os import getenv
from os.path import expanduser, join

CONFIG_FILE = getenv('INGEST_CONFIG', join(
    expanduser('~'), '.pacifica-ingest', 'config.ini'))
CHERRYPY_CONFIG = getenv('INGEST_CPCONFIG', join(
    expanduser('~'), '.pacifica-ingest', 'cpconfig.ini'))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Ingest module."""
from sys import argv as sys_argv
from time import sleep
from threading import Thread
from argparse import ArgumentParser, SUPPRESS
import cherrypy
from pacifica.ingest.rest import Root, error_page_default
from pacifica.ingest.orm import database_setup, update_state
from pacifica.ingest.globals import CHERRYPY_CONFIG


def stop_later(doit=False):
    """Used for unit testing stop after 60 seconds."""
    if not doit:  # pragma: no cover
        return

    def sleep_then_exit():
        """
        Sleep for 60 seconds then call cherrypy exit.

        Hopefully this is long enough for the end-to-end tests to finish
        """
        sleep(90)
        cherrypy.engine.exit()
    sleep_thread = Thread(target=sleep_then_exit)
    sleep_thread.daemon = True
    sleep_thread.start()


def main(argv=None):
    """Main method to start the httpd server."""
    parser = ArgumentParser(description='Run the cart server.')
    parser.add_argument('-c', '--config', metavar='CONFIG', type=str,
                        default=CHERRYPY_CONFIG, dest='config',
                        help='cherrypy config file')
    parser.add_argument('-p', '--port', metavar='PORT', type=int,
                        default=8066, dest='port',
                        help='port to listen on')
    parser.add_argument('-a', '--address', metavar='ADDRESS',
                        default='localhost', dest='address',
                        help='address to listen on')
    parser.add_argument('--stop-after-a-moment', help=SUPPRESS,
                        default=False, dest='stop_later',
                        action='store_true')
    args = parser.parse_args(argv)
    database_setup()
    stop_later(args.stop_later)
    cherrypy.config.update({'error_page.default': error_page_default})
    cherrypy.config.update({
        'server.socket_host': args.address,
        'server.socket_port': args.port
    })
    cherrypy.quickstart(Root(), '/', args.config)


def cmd(argv=None):
    """Command line admin tool for managing ingest."""
    parser = ArgumentParser(description='Admin command line tool.')
    subparsers = parser.add_subparsers(help='sub-command help')
    job_parser = subparsers.add_parser(
        'job', help='job help', description='manage jobs')
    for attr in ['job_id', 'state', 'task', 'task_percent', 'exception']:
        job_parser.add_argument(
            '--{}'.format(attr.replace('_', '-')),
            dest=attr,
            help='set the {}'.format(attr)
        )
    job_parser.set_defaults(func=update_state)
    args = parser.parse_args(argv)
    args_func = args.func
    delattr(args, 'func')
    args_func(**(vars(args)))


if __name__ == '__main__':
    main(sys_argv[1:])



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The WSGI interface module for notifications."""
import cherrypy
from pacifica.ingest.rest import Root, error_page_default
from pacifica.ingest.globals import CHERRYPY_CONFIG

cherrypy.config.update({'error_page.default': error_page_default})
# pylint doesn't realize that application is actually a callable
# pylint: disable=invalid-name
application = cherrypy.Application(Root(), '/', CHERRYPY_CONFIG)
# pylint: enable=invalid-name



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test module for the ingest python code."""
from os.path import join
from time import sleep
from json import loads
import requests


def check_upload_state(job_id, wait):
    """Get the upload state and return results."""
    sleep(wait)
    req = requests.get(
        'http://127.0.0.1:8066/get_state?job_id={}'.format(job_id))
    assert req.status_code == 200
    job_state = loads(req.text)
    return job_state


def try_assert_job_state(job_state, state, task, percent):
    """assert on the job state bits."""
    assert job_state['state'] == state
    assert job_state['task'] == task
    assert int(float(job_state['task_percent'])) == percent


def try_good_move(mdfile, state, task, percent, wait=5):
    """Test the move and see if the state task and percent match."""
    with open(join('test_data', '{}.json'.format(mdfile)), 'r') as filefd:
        req = requests.post(
            'http://127.0.0.1:8066/move',
            data=filefd.read(),
            headers={'content-type': 'application/json'}
        )
        assert req.status_code == 200
        job_id = req.json()['job_id']
        job_state = check_upload_state(job_id, wait)
        try_assert_job_state(job_state, state, task, percent)


def try_good_upload(tarfile, state, task, percent, wait=5):
    """Test the upload and see if the state task and percent match."""
    with open(join('test_data', '{}.tar'.format(tarfile)), 'rb') as filefd:
        req = requests.post(
            'http://127.0.0.1:8066/upload',
            data=filefd,
            headers={
                'Content-Type': 'application/octet-stream'
            }
        )
        assert req.status_code == 200
        job_state = check_upload_state(loads(req.text)['job_id'], wait)
        try_assert_job_state(job_state, state, task, percent)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Index server unit and integration tests."""
from __future__ import absolute_import
import os
from tempfile import mkstemp
from pacifica.ingest.__main__ import cmd
from pacifica.ingest.orm import IngestState, read_state
from ingest_db_setup_test import IngestDBSetup


class TestEntryPoints(IngestDBSetup):
    """Test the entry points for console and gui."""

    def test_job_subcommand(self):
        """Test the job subcommand for updating job state."""
        rwfd, _fname = mkstemp()
        os.close(rwfd)
        IngestState.create(
            job_id=998,
            state='ERROR',
            task='unbundling',
            task_percent=42.3
        )
        IngestState.database_close()
        cmd([
            'job',
            '--task=unbundling',
            '--job-id=998',
            '--state=OK',
            '--task-percent=100.0',
            '--exception=Badness'
        ])
        record = read_state(998)
        self.assertEqual(record.state, 'OK')
        self.assertEqual(record.task_percent, 100.0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test ingest with good uploads of good and bad data."""
from __future__ import print_function, absolute_import
import requests
from common_methods_test import try_good_upload


def test_bad_job_id():
    """Test a bad job ID."""
    req = requests.get('http://127.0.0.1:8066/get_state?job_id=12345')
    assert req.status_code == 404


def test_good_upload():
    """Test the good upload."""
    try_good_upload('good', 'OK', 'ingest metadata', 100)


def test_bad_proposal_upload():
    """Test if the metadata is down."""
    try_good_upload('bad-proposal', 'FAILED', 'Policy Validation', 0)


def test_bad_hashsum_upload():
    """Test if the metadata is down."""
    try_good_upload('bad-hashsum', 'FAILED', 'ingest files', 0)


def test_bad_metadata_upload():
    """Test if the metadata is down."""
    try_good_upload('bad-mimetype', 'FAILED', 'ingest metadata', 0)


def test_bad_json_upload():
    """Test if the metadata is down."""
    try_good_upload('bad-json', 'FAILED', 'load metadata', 0)


def test_bad_tarfile_upload():
    """Test if the metadata is down."""
    try_good_upload('bad-tarfile', 'FAILED', 'open tar', 0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test ingest with good uploads of good and bad data."""
from __future__ import print_function, absolute_import
from common_methods_test import try_good_move


def test_good_move():
    """Test the good move."""
    try_good_move('move-md', 'OK', 'ingest metadata', 100)


def test_bad_file_move():
    """Test the bad file path move."""
    try_good_move('bad-move-md', 'FAILED', 'move files', 0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Configuration reading and validation module."""
from os import getenv
try:
    from ConfigParser import SafeConfigParser
except ImportError:  # pragma: no cover python 2 vs 3 issue
    from configparser import SafeConfigParser
from pacifica.ingest.globals import CONFIG_FILE


def get_config():
    """Return the ConfigParser object with defaults set."""
    configparser = SafeConfigParser()
    configparser.add_section('database')
    configparser.set('database', 'peewee_url', getenv(
        'PEEWEE_URL', 'sqliteext:///db.sqlite3'))
    configparser.read(CONFIG_FILE)
    return configparser



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test ingest with a disabled policy server."""
from __future__ import print_function, absolute_import
from common_methods_test import try_good_upload


def test_bad_policy_upload():
    """Test if the policy server is down."""
    try_good_upload('good', 'FAILED', 'Policy Validation', 0, 10)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test ingest."""
import mock
import peewee
from pacifica.ingest.orm import database_setup, IngestState


@mock.patch.object(IngestState, 'table_exists')
def test_bad_db_connection(mock_is_table_exists):
    """Test a failed db connection."""
    mock_is_table_exists.side_effect = peewee.OperationalError(
        mock.Mock(), 'Error')
    hit_exception = False
    try:
        database_setup(18)
    except peewee.OperationalError:
        hit_exception = True
    assert hit_exception



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Module that contains all the amqp tasks that support the ingest infrastructure."""
from __future__ import absolute_import, print_function
import os
import traceback
import requests
from celery import Celery
from pacifica.ingest.tarutils import open_tar
from pacifica.ingest.tarutils import MetaParser
from pacifica.ingest.tarutils import TarIngester
from pacifica.ingest.tarutils import patch_files
from pacifica.ingest.orm import update_state


INGEST_APP = Celery(
    'ingest',
    broker=os.getenv('BROKER_URL', 'pyamqp://'),
    backend=os.getenv('BACKEND_URL', 'rpc://')
)


class IngestException(Exception):
    """Ingest class exception."""

    pass


def ingest_check_tarfile(job_id, filepath):
    """Check the ingest tarfile and return state or set it properly."""
    update_state(job_id, 'OK', 'open tar', 0)
    tar = open_tar(filepath)
    if tar is None:
        update_state(job_id, 'FAILED', 'open tar',
                     0, 'Failed to open tarfile.')
        raise IngestException()
    update_state(job_id, 'OK', 'open tar', 100)
    return tar


def move_metadata_parser(job_id, metafile):
    """Ingest the metadata and set the state appropriately."""
    update_state(job_id, 'OK', 'load metadata', 0)
    meta = MetaParser()
    meta.read_meta(metafile, job_id)
    update_state(job_id, 'OK', 'load metadata', 100)
    return meta


def ingest_metadata_parser(job_id, tar):
    """Ingest the metadata and set the state appropriately."""
    update_state(job_id, 'OK', 'load metadata', 0)
    meta = MetaParser()
    try:
        meta.load_meta(tar, job_id)
    # pylint: disable=broad-except
    except Exception as ex:
        update_state(job_id, 'FAILED', 'load metadata', 0, str(ex))
        raise IngestException()
    update_state(job_id, 'OK', 'load metadata', 100)
    return meta


def ingest_policy_check(job_id, meta_str):
    """Ingest check to validate metadata at policy."""
    success, exception = validate_meta(meta_str)
    if not success:
        update_state(job_id, 'FAILED', 'Policy Validation', 0, exception)
        raise IngestException()
    update_state(job_id, 'OK', 'Policy Validation', 100)


def move_files(job_id, meta_obj):
    """Move the files to the archive interface."""
    update_state(job_id, 'OK', 'move files', 0)
    try:
        patch_files(meta_obj)
    # pylint: disable=broad-except
    except Exception as ex:
        # rollback files
        stack_dump = traceback.format_exc()
        update_state(job_id, 'FAILED', 'move files', 0,
                     u'{}\n{}'.format(stack_dump, str(ex)))
        raise IngestException()
    update_state(job_id, 'OK', 'move files', 100)


def ingest_files(job_id, ingest_obj):
    """Ingest the files to the archive interface."""
    update_state(job_id, 'OK', 'ingest files', 0)
    try:
        ingest_obj.ingest()
    # pylint: disable=broad-except
    except Exception as ex:
        # rollback files
        stack_dump = traceback.format_exc()
        update_state(job_id, 'FAILED', 'ingest files', 0,
                     u'{}\n{}'.format(stack_dump, str(ex)))
        raise IngestException()
    update_state(job_id, 'OK', 'ingest files', 100)


def ingest_metadata(job_id, meta):
    """Ingest metadata to the metadata service."""
    update_state(job_id, 'OK', 'ingest metadata', 0)
    success, exception = meta.post_metadata()
    if not success:
        # rollback files
        update_state(job_id, 'FAILED', 'ingest metadata', 0, str(exception))
        raise IngestException()
    update_state(job_id, 'OK', 'ingest metadata', 100)


@INGEST_APP.task(ignore_result=False)
def move(job_id, filepath):
    """Move a MD bundle into the archive."""
    try:
        meta = move_metadata_parser(job_id, filepath)
        ingest_policy_check(job_id, meta.meta_str)
        move_files(job_id, meta)
        ingest_metadata(job_id, meta)
        os.unlink(filepath)
    except IngestException:
        return


@INGEST_APP.task(ignore_result=False)
def ingest(job_id, filepath):
    """Ingest a tar bundle into the archive."""
    try:
        tar = ingest_check_tarfile(job_id, filepath)
        meta = ingest_metadata_parser(job_id, tar)
        ingest_obj = TarIngester(tar, meta)
        ingest_policy_check(job_id, meta.meta_str)
        ingest_files(job_id, ingest_obj)
        ingest_metadata(job_id, meta)
        tar.close()
        os.unlink(filepath)
    except IngestException:
        return


def validate_meta(meta_str):
    """Validate metadata."""
    try:
        archivei_server = os.getenv('POLICY_SERVER', '127.0.0.1')
        archivei_port = os.getenv('POLICY_PORT', '8181')
        archivei_url = 'http://{0}:{1}/ingest'.format(
            archivei_server, archivei_port)

        headers = {'content-type': 'application/json'}

        session = requests.session()
        retry_adapter = requests.adapters.HTTPAdapter(max_retries=5)
        session.mount('http://', retry_adapter)
        session.mount('https://', retry_adapter)
        req = session.post(archivei_url, headers=headers, data=meta_str)

        req_json = req.json()
        if req_json['status'] == 'success':
            return True, ''
        return False, '{} ({})'.format(req_json['message'], req_json['traceback'])
    # pylint: disable=broad-except
    except Exception as ex:
        return False, ex
    # pylint: enable=broad-except
